/*
 *
 * Copyright © 2007 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 */
package example.payment.jbricks;

import javax.microedition.lcdui.*;


/**
 * This class drives the games (hence the name
 * engine :). It has all the level data, animates
 * the objects, keeps track of state, and triggers
 * screen redraws.
 */
public class Engine implements Runnable {
    public static final int PATTERN_WIDTH = 11;
    public static final int TITLE = 0;
    public static final int PLAY = 1;
    public static final int OVER = 2;
    public static final int MENU = 3;
    private static final int LAST_KEY_DELTA = 7000;
    private static final int STD = Brick.STANDARD;
    private static final int FIX = Brick.FIXED;
    private static final int SLI = Brick.SLIDE;
    private static final int ZOM = Brick.ZOMBIE;
    private static final int MAX_LEVELS = 6;
    private static final int MAX_LIVES = 6;
    private Ball ball;
    private Brick paddle;
    private BrickList bricks;
    private BrickList oldBricks;
    private Screen screen;
    private int score;
    private int hiScore;
    private int level;
    private int availableLevels;
    private int lives;
    private int availableLives;
    private int state;
    private int key;
    private boolean paused;
    private long lastKeyPress;
    private boolean levelStarted;
    private boolean done;
    private Menu menu;

    // Note that the width of each pattern
    // has to be equal to the PATTERN_WIDTH
    // variable above
    private int[] title_pattern =
        {
            STD, STD, STD, STD, STD, STD, STD, STD, STD, STD, STD, STD, STD, STD, STD, FIX, FIX, FIX,
            STD, STD, STD, STD, STD, STD, STD, STD, STD, STD, FIX, STD, STD, STD, STD, STD, STD, STD,
            STD, STD, STD, FIX, STD, STD, STD, STD, STD, STD, STD, STD, FIX, STD, FIX, STD, STD, STD,
            STD, STD, STD, STD, STD, STD, FIX, STD, STD, STD, STD, STD, STD, STD, STD, STD, STD, STD,
            STD, STD, STD, STD, STD
        };
    private int[][] pattern_list =
        {
            {
                STD, STD, STD, STD, STD, STD, STD, STD, STD, STD, STD, STD, STD, STD, STD, STD, STD,
                STD, STD, STD, STD, STD, STD, STD, STD, STD, STD, STD, STD, STD, STD, STD, STD, STD,
                STD, STD, STD, STD, STD, STD, STD, STD, STD, STD, STD, STD, STD, STD, STD, STD, STD,
                STD, STD, STD, STD, STD, STD, STD, STD, STD, STD, STD, STD, STD, STD, STD, STD, STD,
                STD, STD, STD, STD, STD, STD, STD, STD, STD, STD, STD, STD, STD, STD, STD, STD, STD,
                STD, STD, STD,
            },
            {
                STD, STD, STD, STD, STD, STD, STD, STD, STD, STD, STD, ZOM, SLI, ZOM, ZOM, SLI, ZOM,
                ZOM, SLI, ZOM, ZOM, SLI, ZOM, ZOM, ZOM, ZOM, ZOM, ZOM, ZOM, ZOM, ZOM, ZOM, ZOM, SLI,
                ZOM, ZOM, SLI, ZOM, ZOM, SLI, ZOM, ZOM, SLI, ZOM, ZOM, ZOM, ZOM, ZOM, ZOM, ZOM, ZOM,
                ZOM, ZOM, ZOM, ZOM, ZOM, SLI, ZOM, ZOM, SLI, ZOM, ZOM, SLI, ZOM, ZOM, SLI, ZOM, ZOM,
                ZOM, ZOM, ZOM, ZOM, ZOM, ZOM, ZOM, ZOM, ZOM, SLI, ZOM, ZOM, SLI, ZOM, ZOM, SLI, ZOM,
                ZOM, SLI, ZOM,
            },
            {
                ZOM, ZOM, ZOM, ZOM, ZOM, ZOM, ZOM, ZOM, ZOM, ZOM, ZOM, ZOM, FIX, SLI, FIX, FIX, FIX,
                FIX, FIX, SLI, FIX, ZOM, ZOM, FIX, ZOM, ZOM, ZOM, ZOM, ZOM, ZOM, ZOM, FIX, ZOM, ZOM,
                FIX, ZOM, STD, STD, STD, STD, STD, ZOM, FIX, ZOM, ZOM, FIX, ZOM, ZOM, ZOM, ZOM, ZOM,
                ZOM, ZOM, FIX, ZOM, ZOM, FIX, FIX, FIX, FIX, FIX, FIX, FIX, FIX, FIX, ZOM, ZOM, ZOM,
                ZOM, ZOM, ZOM, ZOM, ZOM, ZOM, ZOM, ZOM, ZOM, STD, STD, STD, STD, STD, STD, STD, STD,
                STD, STD, STD,
            },
            {
                STD, STD, STD, STD, STD, STD, STD, STD, STD, STD, STD, STD, FIX, FIX, FIX, FIX, FIX,
                FIX, FIX, FIX, FIX, STD, STD, FIX, STD, STD, STD, STD, STD, STD, STD, FIX, STD, STD,
                STD, STD, STD, STD, STD, STD, STD, STD, STD, STD, STD, STD, STD, STD, STD, STD, STD,
                STD, STD, STD, STD, STD, FIX, STD, STD, STD, STD, STD, STD, STD, FIX, STD, STD, FIX,
                FIX, FIX, FIX, FIX, FIX, FIX, FIX, FIX, STD, STD, STD, STD, STD, STD, STD, STD, STD,
                STD, STD, STD
            },
            {
                FIX, FIX, FIX, FIX, FIX, ZOM, FIX, FIX, FIX, FIX, FIX, FIX, STD, STD, STD, STD, ZOM,
                STD, STD, STD, STD, FIX, FIX, STD, STD, SLI, SLI, ZOM, SLI, SLI, STD, STD, FIX, STD,
                STD, STD, STD, STD, ZOM, STD, STD, STD, STD, STD, STD, STD, STD, STD, STD, ZOM, STD,
                STD, STD, STD, STD, FIX, STD, STD, STD, STD, SLI, STD, STD, STD, STD, FIX, FIX, STD,
                STD, STD, STD, ZOM, STD, STD, STD, STD, FIX, FIX, FIX, FIX, FIX, FIX, ZOM, FIX, FIX,
                FIX, FIX, FIX
            },
            {
                SLI, SLI, SLI, SLI, SLI, SLI, SLI, SLI, SLI, SLI, SLI, SLI, ZOM, ZOM, ZOM, ZOM, ZOM,
                ZOM, ZOM, ZOM, ZOM, FIX, SLI, ZOM, ZOM, SLI, ZOM, ZOM, ZOM, SLI, ZOM, ZOM, FIX, SLI,
                ZOM, ZOM, ZOM, STD, STD, STD, ZOM, ZOM, ZOM, FIX, SLI, ZOM, ZOM, ZOM, STD, STD, STD,
                ZOM, ZOM, ZOM, FIX, SLI, ZOM, ZOM, SLI, ZOM, ZOM, ZOM, SLI, ZOM, ZOM, FIX, SLI, ZOM,
                ZOM, ZOM, ZOM, ZOM, ZOM, ZOM, ZOM, ZOM, FIX, FIX, FIX, FIX, FIX, FIX, FIX, FIX, FIX,
                FIX, FIX, FIX
            }
        };

    public Engine(Screen screen) {
        int i;

        menu = new Menu();
        menu.setEngine(this);

        this.screen = screen;
        screen.setEngine(this);

        paddle = new Brick(null, (Screen.width / 2) - (Brick.WIDTH / 2), (Screen.height * 8) / 10,
                -1, 2);
        paddle.setColor(ThreeDColor.lightGray);
        paddle.width = Screen.width / 8;
        paddle.height = Brick.HEIGHT;

        ball = new Ball((Screen.width / 2) - (Brick.WIDTH / 4),
                ((Screen.height * 8) / 10) - (2 * Ball.RADIUS), 2 * Math.max(1, Screen.width / 80),
                2 * -Math.max(1, Screen.width / 80));

        availableLevels = 1;
        availableLives = 1;

        level = 0;
        state = TITLE;
        done = false;
        bricks = new BrickList(title_pattern, PATTERN_WIDTH, -1);
        lastKeyPress = System.currentTimeMillis();

        Thread runner = new Thread(this);
        runner.start();
    }

    private void reset() {
        lives = availableLives;
        score = 0;
        startLevel();
    }

    private void restartLevel() {
        if (state == PLAY) {
            paused = true;
        }

        synchronized (this) {
            levelStarted = true;
        }

        paddle.moveTo((Screen.width / 2) - (Brick.WIDTH / 2), (Screen.height * 8) / 10);
        ball.moveTo((Screen.width / 2) - (Brick.WIDTH / 4),
            ((Screen.height * 8) / 10) - (2 * Ball.RADIUS));
        ball.setSteps(Math.max(1, Screen.width / 80), -Math.max(1, Screen.width / 80));

        screen.repaint();
        screen.serviceRepaints();
    }

    private void startLevel() {
        bricks = new BrickList(pattern_list[level], PATTERN_WIDTH, level);
        restartLevel();
    }

    private void nextLevel() {
        level++;

        if ((level == availableLevels) || (level == pattern_list.length)) {
            level = 0;
        }

        startLevel();
    }

    public void keyPressed(int keycode, int gameAction) {
        key = gameAction;
        lastKeyPress = System.currentTimeMillis();

        if ((state == OVER) || (state == TITLE)) {
            state = MENU;
            bricks = new BrickList(title_pattern, PATTERN_WIDTH, -1);
        } else if ((state == PLAY) &&
                ((key == Canvas.LEFT) || (key == Canvas.RIGHT) || (key == Canvas.FIRE))) {
            paused = false;
        }
    }

    public void keyReleased(int keycode, int gameAction) {
        key = 0;
    }

    public void getState(EngineState engineState) {
        engineState.bricks = bricks;
        engineState.ball = ball;
        engineState.paddle = paddle;
        engineState.state = state;
        engineState.score = score;
        engineState.hiScore = hiScore;
        engineState.lives = lives;
        engineState.menu = menu;
    }

    protected void startGame(int level) {
        this.level = level;
        state = PLAY;
        paused = false;
        reset();
    }

    public void setLastKeyPressed(long time) {
        lastKeyPress = time;
    }

    public synchronized boolean levelStarted() {
        boolean x = levelStarted;
        levelStarted = false;

        return x;
    }

    public void stop() {
        done = true;
    }

    public void setPaused(boolean pause) {
        paused = pause;
    }

    public void run() {
        boolean recentCollision;
        long then;
        int px;
        int pw;
        int delta;
        int paddleSpeed = 0;

        recentCollision = false;
        then = System.currentTimeMillis();

        while (!done) {
            if (((state == TITLE) || (state == OVER)) &&
                    ((System.currentTimeMillis() - lastKeyPress) > LAST_KEY_DELTA)) {
                state = MENU;
                lastKeyPress = System.currentTimeMillis();
                bricks = new BrickList(title_pattern, PATTERN_WIDTH, -1);
            }

            px = paddle.x;
            pw = paddle.width;

            if ((state == PLAY) && !paused) {
                if (key == Canvas.LEFT) {
                    paddleSpeed = Math.min(-1, -Brick.STEP);
                } else if (key == Canvas.RIGHT) {
                    paddleSpeed = Math.max(1, Brick.STEP);
                } else {
                    paddleSpeed = 0;
                }

                if (((paddleSpeed < 0) && (px > 0)) ||
                        ((paddleSpeed > 0) && ((px + pw) < Screen.width))) {
                    paddle.moveBy(paddleSpeed, 0);
                }

                ball.move();

                if ((ball.x <= 0) || (ball.x >= (Screen.width - ball.width - 1))) {
                    ball.bounceHorizontal();
                }

                if (ball.y <= 0) {
                    ball.bounceVertical();
                }

                if (ball.intersects(paddle)) {
                    if (!recentCollision) {
                        ball.bounce(paddle);
                    }

                    recentCollision = true;
                } else {
                    recentCollision = false;
                }

                score += bricks.checkForCollision(ball);

                if ((state == PLAY) && (score >= hiScore)) {
                    hiScore = score;
                }

                if (bricks.isClean()) {
                    nextLevel();
                }

                if (ball.y >= (paddle.y + paddle.height)) {
                    if (state == PLAY) {
                        lives--;
                    }

                    if (lives < 0) {
                        state = OVER;
                        level = 0;
                    }

                    restartLevel();
                }
            }

            screen.repaint();

            delta = (int)(System.currentTimeMillis() - then);

            if (delta < 30) {
                try {
                    Thread.sleep(30 - delta);
                } catch (InterruptedException e) {
                }
            }

            then = System.currentTimeMillis();
        }
    }

    protected void setState(int state) {
        this.state = state;
    }

    protected void increaseNumOfLevels(int inc) {
        availableLevels += inc;
    }

    protected void increaseNumOfLives(int inc) {
        availableLives += inc;
        lives += inc;
    }

    protected int getAvailableLevels() {
        return availableLevels;
    }

    protected int getAvailableLives() {
        return availableLives + 1;
    }

    protected boolean canBuyLevels(int count) {
        return (availableLevels + count) < MAX_LEVELS;
    }

    protected boolean canBuyLives(int count) {
        return (availableLives + count) < MAX_LIVES;
    }

    protected void showMenu() {
        state = MENU;
        oldBricks = bricks;
        bricks = new BrickList(title_pattern, PATTERN_WIDTH, -1);
        paused = true;
        screen.repaint();
    }

    protected void resumeGame() {
        if (state == PLAY) {
            paused = true;
        }

        synchronized (this) {
            levelStarted = true;
        }

        if (oldBricks != null) {
            bricks = oldBricks;
        }

        screen.repaint();
        screen.serviceRepaints();
    }
}
