/*
 *
 * Copyright © 2007 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 */
package example.payment.jbricks;

import java.util.Date;

import javax.microedition.lcdui.*;
import javax.microedition.midlet.MIDlet;
import javax.microedition.payment.TransactionListener;
import javax.microedition.payment.TransactionListenerException;
import javax.microedition.payment.TransactionModule;
import javax.microedition.payment.TransactionModuleException;
import javax.microedition.payment.TransactionRecord;


/** main wrapper */
public class Main extends MIDlet implements TransactionListener, CommandListener, Runnable {
    private static final int FEATURE_1_LIFE = 0;
    private static final int FEATURE_3_LIVES = 1;
    private static final int FEATURE_1_LEVEL = 2;
    private static final int FEATURE_3_LEVELS = 3;
    private Command exit = new Command("Exit", Command.EXIT, 0);
    private Screen screen;
    private Engine engine;
    private TransactionModule txModule;
    private int feature = 0;
    private boolean enableTranListenerNull = false;
    private String title = "";
    private String description = "";

    public Main() {
        screen = new Screen(this);
        engine = new Engine(screen);

        screen.setCommandListener(this);

        try {
            txModule = new TransactionModule(this);
            txModule.setListener(this);
            txModule.deliverMissedTransactions();
            restoreBoughtFeatures();
        } catch (TransactionListenerException tle) {
            tle.printStackTrace();
        } catch (TransactionModuleException tme) {
            tme.printStackTrace();
        }
    }

    public void showCommandExit(boolean show) {
        if (show) {
            screen.addCommand(exit);
        } else {
            screen.removeCommand(exit);
        }
    }

    public void startApp() {
        Display.getDisplay(this).setCurrent(screen);
        engine.setPaused(false);
    }

    public void pauseApp() {
        engine.setPaused(true);
    }

    public void destroyApp(boolean unc) {
        engine.stop();
        Display.getDisplay(this).setCurrent(null);
        notifyDestroyed();
    }

    public void commandAction(Command cmd, Displayable dis) {
        if (cmd == exit) {
            engine.showMenu();
            showCommandExit(false);
        }
    }

    public void processed(TransactionRecord record) {
        switch (record.getState()) {
        case TransactionRecord.TRANSACTION_SUCCESSFUL:

            switch (record.getFeatureID()) {
            case FEATURE_1_LIFE:
                engine.increaseNumOfLives(1);

                break;

            case FEATURE_3_LIVES:
                engine.increaseNumOfLives(3);

                break;

            case FEATURE_1_LEVEL:
                engine.increaseNumOfLevels(1);

                break;

            case FEATURE_3_LEVELS:
                engine.increaseNumOfLevels(3);

                break;
            }

            break;

        default:
        }
    }

    public void run() {
        try {
            txModule.setListener(this);
            txModule.process(feature, title, description);

            if (enableTranListenerNull) {
                txModule.setListener(null);
            }
        } catch (TransactionListenerException tle) {
            System.err.println("Transaction Listener is not set");
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    protected void buyLife(int count) {
        title = "Buy Life";

        if (count == 1) {
            feature = FEATURE_1_LIFE;
            description = "You are able to increase number of " + "your lives by 1 life.";
        } else {
            feature = FEATURE_3_LIVES;
            description = "You are able to increase number of " + "your lives by 3 lives.";
        }

        if (txModule != null) {
            new Thread(this).start();
        }
    }

    protected void buyLevel(int count) {
        title = "Buy Level";

        if (count == 1) {
            feature = FEATURE_1_LEVEL;
            description = "You are able to increases number of " +
                "levels you can access by 1 level.";
        } else {
            feature = FEATURE_3_LEVELS;
            description = "You are able to increases number of " +
                "levels you can access by 3 levels.";
        }

        if (txModule != null) {
            new Thread(this).start();
        }
    }

    protected void setTransactionListenerNull(boolean b) {
        enableTranListenerNull = b;
    }

    protected String[] getHistory() {
        TransactionRecord[] record = txModule.getPastTransactions(6);
        String[] stringRecord = null;

        if (record != null) {
            String feature = "";
            String when = "";
            Date date = new Date();
            stringRecord = new String[record.length];

            for (int i = 0; i < record.length; i++) {
                switch (record[i].getState()) {
                case TransactionRecord.TRANSACTION_FAILED:
                    feature = "Failed ";

                    break;

                case TransactionRecord.TRANSACTION_REJECTED:
                    feature = "Rejected ";

                    break;

                default:
                    feature = "";
                }

                switch (record[i].getFeatureID()) {
                case FEATURE_1_LIFE:
                    feature += "1 life";

                    break;

                case FEATURE_3_LIVES:
                    feature += "3 lives";

                    break;

                case FEATURE_1_LEVEL:
                    feature += "1 level";

                    break;

                case FEATURE_3_LEVELS:
                    feature += "3 levels";
                }

                date.setTime(record[i].getFinishedTimestamp());
                when = date.toString();
                stringRecord[i] = feature + " on " + when.substring(0, when.lastIndexOf(':'));
            }
        }

        return stringRecord;
    }

    private void restoreBoughtFeatures() {
        TransactionRecord[] record = txModule.getPastTransactions(10);

        if (record != null) {
            for (int i = 0; i < record.length; i++) {
                processed(record[i]);
            }
        }
    }
}
