/*
 *
 * Copyright © 2007 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 */
package example.payment.jbricks;

import javax.microedition.lcdui.*;
import javax.microedition.midlet.MIDlet;


/**
 * This class handles and coordinates most of the
 * drawing for this game. It knows how to draw the
 * splash and game over screen, as well as the
 * actual games screen. It also forwards 'cooked'
 * key events to the Engine class.
 */
public class Screen extends Canvas {
    public static int width;
    public static int height;
    private static final int TL_ANCHOR = Graphics.TOP | Graphics.LEFT;
    private static final int TH_ANCHOR = Graphics.TOP | Graphics.HCENTER;
    public static Graphics GRAPHICS;
    public static final int BACKGROUND = ThreeDColor.gray.getRGB();
    private Font fontSmall;
    private Font fontMedium;
    private Font fontLarge;
    private Font font;
    private Image buf;
    private int lastState;
    private int lastScore;
    private int lastLives;
    private Engine engine;
    private EngineState state;
    private int fontHeight;
    private MIDlet midlet;

    public Screen(MIDlet midlet) {
        width = getWidth();
        height = getHeight();

        fontSmall = Font.getFont(Font.FACE_SYSTEM, Font.STYLE_PLAIN, Font.SIZE_SMALL);
        fontMedium = Font.getFont(Font.FACE_SYSTEM, Font.STYLE_PLAIN, Font.SIZE_MEDIUM);
        fontLarge = Font.getFont(Font.FACE_SYSTEM, Font.STYLE_PLAIN, Font.SIZE_LARGE);

        font = (height < 150) ? fontSmall : fontMedium;
        fontHeight = font.getHeight();

        buf = Image.createImage(width, height - (fontHeight + 5));
        GRAPHICS = buf.getGraphics();
        state = new EngineState();

        lastState = -1;
        lastScore = -1;
        lastLives = -1;

        this.midlet = midlet;
    }

    public void setEngine(Engine engine) {
        this.engine = engine;
        engine.getState(state);
        state.menu.setMIDlet(midlet);
    }

    public void keyPressed(int keycode) {
        if (state.state == Engine.MENU) {
            engine.setLastKeyPressed(System.currentTimeMillis());
            state.menu.keyPressed(getGameAction(keycode));
        } else {
            engine.keyPressed(keycode, getGameAction(keycode));
        }
    }

    public void keyReleased(int keycode) {
        if (state.state != Engine.MENU) {
            engine.keyReleased(keycode, getGameAction(keycode));
        }
    }

    private void paintBackground(Graphics g) {
        g.setColor(BACKGROUND);
        g.fillRect(0, 0, width, height);
    }

    private void paintTitle(Graphics g) {
        engine.getState(state);

        if (lastState != Engine.TITLE) {
            paintBackground(GRAPHICS);
            state.bricks.paintShadow(GRAPHICS);
            state.bricks.paint(GRAPHICS);
        }

        g.drawImage(buf, 0, 0, TL_ANCHOR);
        g.setColor(BACKGROUND);
        g.fillRect(0, height - (fontHeight + 5), width, fontHeight + 5);

        g.setFont(fontLarge);

        g.setColor(ThreeDColor.black.getRGB());
        g.drawString("Bricks", (width / 2) + 1, (height / 2) + 1, TH_ANCHOR);

        g.setColor(ThreeDColor.red.getRGB());
        g.drawString("Bricks", width / 2, height / 2, TH_ANCHOR);
    }

    private void paintOver(Graphics g) {
        paintBackground(g);

        g.setFont(fontLarge);

        g.setColor(ThreeDColor.black.getRGB());
        g.drawString("Game", (width / 2) + 1, (height / 2) - 10 + 1, TH_ANCHOR);
        g.drawString("Over", (width / 2) + 1, (height / 2) + 10 + 1, TH_ANCHOR);

        g.setColor(ThreeDColor.red.getRGB());
        g.drawString("Game", width / 2, (height / 2) - 10, TH_ANCHOR);
        g.drawString("Over", width / 2, (height / 2) + 10, TH_ANCHOR);
    }

    public void paintMenu(Graphics g) {
        engine.getState(state);

        if (lastState != Engine.MENU) {
            paintBackground(GRAPHICS);
            state.bricks.paintShadow(GRAPHICS);
            state.bricks.paint(GRAPHICS);
        }

        g.drawImage(buf, 0, 0, TL_ANCHOR);
        g.setColor(BACKGROUND);
        g.fillRect(0, height - (fontHeight + 5), width, fontHeight + 5);

        g.setFont(fontLarge);

        g.setColor(ThreeDColor.black.getRGB());

        for (int i = 0; i < state.menu.getItems().length; i++) {
            g.drawString(state.menu.getItems()[i].getText(), (width / 2) + 1,
                (height / 3) + 30 + 1 + (20 * i), TH_ANCHOR);
        }

        for (int i = 0; i < state.menu.getItems().length; i++) {
            if (state.menu.getItems()[i].isEnabled()) {
                g.setColor(ThreeDColor.purple.getRGB());
            } else {
                g.setColor(ThreeDColor.gray.getRGB());
            }

            g.drawString(state.menu.getItems()[i].getText(), width / 2,
                (height / 3) + 30 + (20 * i), TH_ANCHOR);
        }

        g.setColor(ThreeDColor.pink.getRGB());

        g.drawString(state.menu.getItems()[state.menu.getSelected()].getText(), width / 2,
            (height / 3) + 30 + (20 * state.menu.getSelected()), TH_ANCHOR);
    }

    public void paint(Graphics g) {
        boolean full_repaint = engine.levelStarted();

        engine.getState(state);
        g.setFont(font);

        if (state.state == Engine.TITLE) {
            paintTitle(g);
        } else if (state.state == Engine.MENU) {
            paintMenu(g);
        } else if (state.state == Engine.OVER) {
            paintOver(g);
            state.menu.resetSelected();
            state.menu.showResumeItem(false);
        } else {
            if (full_repaint) {
                paintBackground(GRAPHICS);

                state.bricks.paintShadow(GRAPHICS);
                state.bricks.paint(GRAPHICS);
            }

            g.drawImage(buf, 0, 0, TL_ANCHOR);

            if (state.state == Engine.PLAY) {
                if ((state.score != lastScore) || full_repaint) {
                    g.setColor(BACKGROUND);
                    g.fillRect(width / 3, height - (fontHeight + 5), (width * 2) / 3, fontHeight +
                        5);

                    g.setColor(ThreeDColor.black.getRGB());
                    g.drawString("S: " + state.score, ((width * 1) / 3) + 1,
                        height - (fontHeight + 3), TL_ANCHOR);
                    g.drawString("H: " + state.hiScore, ((width * 2) / 3) + 1,
                        height - (fontHeight + 3), TL_ANCHOR);
                    g.setColor(ThreeDColor.orange.getRGB());
                    g.drawString("S: " + state.score, (width * 1) / 3, height - (fontHeight + 5),
                        TL_ANCHOR);
                    g.drawString("H: " + state.hiScore, (width * 2) / 3, height - (fontHeight + 5),
                        TL_ANCHOR);
                }

                if ((state.lives != lastLives) || full_repaint) {
                    g.setColor(BACKGROUND);
                    g.fillRect(0, height - (fontHeight + 5), width / 3, fontHeight + 5);

                    for (int i = 0; i < state.lives; i++) {
                        g.setColor(ThreeDColor.black.getRGB());
                        g.fillArc(BrickList.XOFFSET + 1 + (3 * Ball.RADIUS * i),
                            height - (fontHeight - 1), Math.max(4, height / 50),
                            Math.max(4, height / 50), 0, 360);

                        g.setColor(ThreeDColor.red.getRGB());
                        g.fillArc(BrickList.XOFFSET + (3 * Ball.RADIUS * i), height - fontHeight,
                            Math.max(4, height / 50), Math.max(4, height / 50), 0, 360);
                    }
                }
            }

            state.ball.paintShadow(g);
            state.paddle.paintShadow(g);

            state.ball.paint(g);
            state.paddle.paint(g);
        }

        lastState = state.state;
        lastScore = state.score;
        lastLives = state.lives;
    }
}
