/*
 *
 * Copyright © 2007 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 */
package example.payment.jbricks;


/**
 * This class provides support for pseudo-3D coloring.
 * Each of the defined colors can used as a base color,
 * and the brighter() and darker() methods can be used
 * to retrieve the appropriately highlighted or shadowed
 * version of that color. The getRGB() method is used to
 * retrieve the actual RGB value of the color, the returned
 * value can be passed to Graphics.setColor().
 */
public class ThreeDColor {
    public static ThreeDColor lightGray = new ThreeDColor(0xc0c0c0);
    public static ThreeDColor lightPurple = new ThreeDColor(0xb000ff);
    public static ThreeDColor black = new ThreeDColor(0x000000);
    public static ThreeDColor gray = new ThreeDColor(0x808080);
    public static ThreeDColor blue = new ThreeDColor(0x0000ff);
    public static ThreeDColor red = new ThreeDColor(0xff0000);
    public static ThreeDColor yellow = new ThreeDColor(0xffff00);
    public static ThreeDColor white = new ThreeDColor(0xffffff);
    public static ThreeDColor orange = new ThreeDColor(0xffc800);
    public static ThreeDColor green = new ThreeDColor(0x00ff00);
    public static ThreeDColor purple = new ThreeDColor(0x8000c0);
    public static ThreeDColor darkCyan = new ThreeDColor(0x00b2b2);
    public static ThreeDColor pink = new ThreeDColor(0xffafaf);
    public static ThreeDColor darkGray = new ThreeDColor(0x404040);
    public static ThreeDColor darkGreen = new ThreeDColor(0x00b200);
    public static ThreeDColor darkOrange = new ThreeDColor(0xb28c00);
    public static ThreeDColor darkPurple = new ThreeDColor(0x500080);
    public static ThreeDColor darkRed = new ThreeDColor(0xb20000);
    private int rgb;

    private ThreeDColor(int rgb) {
        this.rgb = rgb;
    }

    public ThreeDColor brighter() {
        if (this == lightGray) {
            return white;
        }

        if (this == gray) {
            return lightGray;
        }

        if (this == darkGreen) {
            return green;
        }

        if (this == blue) {
            return darkCyan;
        }

        if (this == red) {
            return pink;
        }

        if (this == yellow) {
            return white;
        }

        if (this == orange) {
            return yellow;
        }

        if (this == green) {
            return yellow;
        }

        if (this == purple) {
            return lightPurple;
        }

        return white;
    }

    public ThreeDColor darker() {
        if (this == lightGray) {
            return gray;
        }

        if (this == gray) {
            return darkGray;
        }

        if (this == darkGreen) {
            return darkGray;
        }

        if (this == blue) {
            return darkGray;
        }

        if (this == red) {
            return darkRed;
        }

        if (this == yellow) {
            return orange;
        }

        if (this == orange) {
            return darkOrange;
        }

        if (this == green) {
            return darkGreen;
        }

        if (this == purple) {
            return darkPurple;
        }

        return darkGray;
    }

    public int getRGB() {
        return rgb;
    }
}
